//=============================================================================
// Mutator Adder - Server Actor
//
// Add mutators via ini file instead of server command line options.
//
// Author: }TCP{Wolf
//  Date: 2010-10-11
// CodeSchool demonstration mod.
//
// This server actor spawns all configured mutators and adds them the same way
// the command line option would (BaseMutator.AddMutator(...)).
// Under normal circumstances, a mod like this would be utterly pointless,
// however, there are instances where you may not be able change the command
// line of your server, or only with difficulty. Cheap remote hosters are one
// such "circumstance".
//
// INSTALLATION
// Copy MutatorAdderSA.u and MutatorAdderSA.int to your System directory.
// Add the following entry in your Unreal.ini where the server packages and
// server actors are [Engine.GameEngine]
// ServerActors=MutatorAdderSA.MutatorAdderSA
//
// CONFIGURATION
// Settings are stored in your main ini (Unreal.ini) in the section
// [MutatorAdderSA.MutatorAdderSA] - you can open the file
// Unreal.ini.ExampleConfig.txt for a copy/paste/edit example for you to start
// off with. The settings in detail are...
// bDebug - True/False(default) - set to true for increased log
//                                output/verbosity
// bPollCommands - True(default)/False - while true, keeps track if you enter
//                      ingame commands for this mutator (see next chapter),
//                      can be turned off.
// Mutatorlist[0-63] - Fully qualified mutator class as PACKAGE.CLASS
//                     If prefixed with "DISABLED:" the mutator will not load.
//
// COMMANDS
// If the setting "bPollCommands" is true (it is by default), MutatorAdderSA
// will check for ingame commands. You issue ingame commands either by typing
// them on the server console or by logging into your server as player/spec
// with admin rights and issuing these commands prefixed by the word "admin".
//
// All in game commands start with this (server console)
// set mutatoraddersa cmd
//
// If you are an admin player/spec in the server, you would type
// admin set mutatoraddersa cmd
//
// "cmd" can be
// "list" - when this command is issued, the mutator will list its current
//          mutator list to you - faulty, running and disabled ones. Blank
//          configuration entries are NOT shown.
// a number - if you issue a number from 0 to 63, the mutator configured at
//          that indicated slot will be toggled on/off (disabled/enabled).
//          Obviously, for the changes to take effect, a map change is
//          required.
//
// This mod was inspired by oZ//Redrum.
//=============================================================================

class MutatorAdderSA extends Info;

// static variables
var const String disabledPrefix; // defined constant "DISABLED:"

// configuration
var() config bool bDebug;	// set true if you need some additional debug infos when mutators are being loaded
var() config bool bPollCommands; // set to false to disable runtime commands
var() config String Mutatorlist[64]; // list of mutators to spawn, prefixed with "DISABLED:" to indicate a disabled mutator

// runtime
var String cmd;	// command, a number or "list"
var String MutatorlistRemark[ArrayCount(Mutatorlist)];	// store a remark like "failed to load" if needed...

// Which event to use?
// The Unreal engine calls a number of events on all actors to set them up.
// The BaseMutator is spawned in GameInfo.initGame(), so the next best event
// to intercept is PreBeginPlay(). Hopefully, this will minimize any side
// effects a BELATED mutator spawn may have in some mods.

event PreBeginPlay()
{
	local int i, counter;
	local Mutator m;

	log("MutatorAdderSA starting up...",'MUTADD');
	Super.PreBeginPlay();

	counter = 0;
	for (i=0; i<ArrayCount(Mutatorlist); i++)
	{
		m = loadMutator(Mutatorlist[i]);
		if (m != none)
		{
			Level.Game.BaseMutator.AddMutator(m);
			log("Added: "$m,'ADDMUT');
			MutatorlistRemark[i] = "ADDED";
			counter++;
		}
	}

	if (bPollCommands)
		SetTimer(1, true);	// tell engine to keep polling for commands every full second

	log("MutatorAdderSA is finished. Total mutators added: "$counter$". In-game command aware: "$bPollCommands,'MUTADD');
}


// Convenience function to load a specific mutator class by name
// Differentiated ways to resolve a particular class could be implemented here
// in addition to the standard "DynamicLoadObject(...)" approach if desired.
// An optional out parameter can be given if load failures should store
// away the reason for the fail(s).
function Mutator loadMutator(String mutName, optional out String remark)
{
	local class<Mutator> MClass;
	local int l;

	// sanity check - empty argument?
	if (mutName == "")
		return none;

	// sanity check - disabled name?
	if (InStr(mutName,disabledPrefix) == 0)
	{
		log("Not loaded disabled mutator: "$Right(mutName,len(mutName)-len(disabledPrefix)),'MUTADD');
		remark = "DISABLED";
		return none;
	}

	// first attempt: try loading mutator class by name as given
	if (bDebug) log("Trying to load: "$mutName,'MUTADD');
	MClass = class<Mutator>(DynamicLoadObject(mutName, class'Class'));
	if (MClass == none) // load failed... lets try something else...
	{
		// was the full package specified or just one word?
		l = InStr(mutName,".");
		if (l == -1)	// nope
		{
			// often, mutators in single packages are named like the package itself,
			// like this one, it would be "MutatorAdderSA.MutatorAdderSA"
			// so trying PACKAGENAME.CLASSNAME where PACKAGENAME == CLASSNAME can make sense
			if (bDebug) log("Trying to load: "$mutName$"."$mutName);
			MClass = class<Mutator>(DynamicLoadObject(mutName$"."$mutName, class'Class'));
			if (MClass != none) remark = "AUTORES";	// store away how it was loaded
		}

		/* **********************************************************************
			TODO: AutoResolve on loadfail strategies
			Other naming/autoresolve schemes can be added here!
			Meaningful examples would be listing standard packages, for example,
			in combination of the mutator name - such as UnrealShare, UnrealI
			or even a freely configurable package list that should be
			"investigated".
		********************************************************************** */
	}

	// all class loading attempts done... do we have a class?
	if (MClass == none)	// hmm no :(
	{
		log("Failed to load mutator with name: "$mutName,'MUTADD');
		return none;
	}

	log("Loaded successfully: "$mutName,'MUTADD');
	if (remark == "")
		remark = "LOADED";	// class was loaded ok, does not mean was added (yet)
	return Spawn(MClass);
}


// Timer event is used to check if a command was entered.
// Polling is not the best of programmatical concepts but still valid if
// proper "event interception" can not be implemented. A one second poll
// with a sanity check in front of the entire code if anything needs to be
// done at all is very very very VERY harmless towards CPU cycle waste...
// Just check out some pawn mods existing in the world and what they are
// doing all in tick(..) .... me shudders...
event Timer()
{
	local int i, counter;
	local String s;	// no StringBuilder / StringBuffer exists in UScript :(

	if (cmd == "") return;	// nothing todo as no command was entered...

	// list will display all currently configured mutators and wether or not they are running
	if (cmd == "list")
	{
		counter = 0;
		for (i=0; i<ArrayCount(Mutatorlist); i++)	// go through entire mutatorlist
		{
			if (Mutatorlist[i] != "")	// this list entry contains a config
			{
				// assemble a line of text (this is where a buffered StringAppender would be handy)....
				s = ""$i;
				if (MutatorlistRemark[i] != "")
					s = s$" ["$MutatorlistRemark[i]$"]";
				s = s@Mutatorlist[i];
				// ....and print it
				msgadmin(s);
				counter++;
			}
		}	// end for
		if (counter == 0)
			msgadmin("MutatorAdderSA currently has no mutators configured.");
	}	// end list

	// was a number specified?
	i = int(cmd);
	if ((i > 0) || (cmd == "0"))	// yes, a number was specified...
	{
		// note that int(s) will always return 0 for every string it cannot parse as
		// integer, so for the value zero, the explicit string "0" is checked!

		// sanity check
		if (i > ArrayCount(Mutatorlist))
			msgadmin("Specified entry out of range");
		else
		{
			if (InStr(Mutatorlist[i],disabledPrefix) == 0)	// this particular entry is currently disabled, enable it (remove the disabled prefix)
			{
				Mutatorlist[i] = Right(Mutatorlist[i],len(Mutatorlist[i])-len(disabledPrefix));
				msgadmin("Enabled "$Mutatorlist[i]);
			}
			else	// entry is currently enabled, disable it (add the disabled prefix)
			{
				msgadmin("Disabled "$Mutatorlist[i]);
				Mutatorlist[i] = disabledPrefix$Mutatorlist[i];
			}
			// save changed configuration
			saveconfig();
		}
	}

	// clear the command again, otherwise it gets executed every second...
	cmd = "";
}


// Convenience function, send a message to all admins currently in the server
// and also log it in the system log
function msgadmin(String what)
{
	local Pawn P;

	log(what,'MUTADD');

	for(P = Level.Pawnlist; P != none; P = P.nextPawn)
		if (P.IsA('PlayerPawn') && Playerpawn(P).bAdmin)
			P.ClientMessage(what);
}



defaultproperties
{
	bDebug=False
	bPollCommands=True
	disabledPrefix="DISABLED:"
}
